-- Test payment_account_card
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(39);

set search_path to numerus, auth, public;

select has_table('payment_account_card');
select has_pk('payment_account_card');
select table_privs_are('payment_account_card', 'guest', array []::text[]);
select table_privs_are('payment_account_card', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_account_card', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_account_card', 'authenticator', array []::text[]);

select col_is_fk('payment_account_card', array['payment_account_id', 'payment_account_type']);
select fk_ok('payment_account_card', array['payment_account_id', 'payment_account_type'], 'payment_account', array['payment_account_id', 'payment_account_type']);

select has_column('payment_account_card', 'payment_account_id');
select col_is_pk('payment_account_card', 'payment_account_id');
select col_type_is('payment_account_card', 'payment_account_id', 'integer');
select col_not_null('payment_account_card', 'payment_account_id');
select col_hasnt_default('payment_account_card', 'payment_account_id');

select has_column('payment_account_card', 'payment_account_type');
select col_type_is('payment_account_card', 'payment_account_type', 'text');
select col_not_null('payment_account_card', 'payment_account_type');
select col_has_default('payment_account_card', 'payment_account_type');
select col_default_is('payment_account_card', 'payment_account_type', 'card');

select has_column('payment_account_card', 'last_four_digits');
select col_type_is('payment_account_card', 'last_four_digits', 'text');
select col_not_null('payment_account_card', 'last_four_digits');
select col_hasnt_default('payment_account_card', 'last_four_digits');

select has_column('payment_account_card', 'expiration_date');
select col_type_is('payment_account_card', 'expiration_date', 'date');
select col_not_null('payment_account_card', 'expiration_date');
select col_hasnt_default('payment_account_card', 'expiration_date');


set client_min_messages to warning;
truncate payment_account_card cascade;
truncate payment_account cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into payment_account (payment_account_id, company_id, payment_account_type, name)
values ( 6, 2, 'bank', 'Bank 2')
     , ( 7, 2, 'card', 'Credit card 2')
     , ( 8, 2, 'cash', 'Cash 2')
     , ( 9, 2, 'other', 'Other 2')
     , (10, 4, 'card', 'Card 4')
;

insert into payment_account_card (payment_account_id, last_four_digits, expiration_date)
values ( 7, '1234', '2024-07-09')
     , (10, '4321', '2025-09-07')
;

prepare payment_account_data as
select company_id, last_four_digits
from payment_account
join payment_account_card using (payment_account_id)
order by payment_account_id, last_four_digits;

set role invoicer;
select is_empty('payment_account_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'payment_account_data',
	$$ values (2, '1234')
	$$,
	'Should only list payment_accounts of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'payment_account_data',
	$$ values (4, '4321')
	$$,
	'Should only list payment_accounts of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'payment_account_data',
	'42501', 'permission denied for table payment_account',
	'Should not allow select to guest users'
);
reset role;

select throws_ok( $$
	insert into payment_account_card (payment_account_id, payment_account_type, last_four_digits, expiration_date)
	values (6, 'bank', '1111', '2025-05-05')
	$$,
       '23514', 'new row for relation "payment_account_card" violates check constraint "payment_account_type_is_card"',
	'Should not allow payment accounts of type bank'
);

select throws_ok( $$
	insert into payment_account_card (payment_account_id, payment_account_type, last_four_digits, expiration_date)
	values (8, 'cash', '1111', '2025-05-05')
	$$,
       '23514', 'new row for relation "payment_account_card" violates check constraint "payment_account_type_is_card"',
	'Should not allow payment accounts of type cash'
);

select throws_ok( $$
	insert into payment_account_card (payment_account_id, payment_account_type, last_four_digits, expiration_date)
	values (9, 'other', '1111', '2025-05-05')
	$$,
       '23514', 'new row for relation "payment_account_card" violates check constraint "payment_account_type_is_card"',
	'Should not allow payment accounts of type other'
);

select throws_ok (
	$$ update payment_account_card set last_four_digits = 'a234' where payment_account_id = 7 $$,
	'23514', 'new row for relation "payment_account_card" violates check constraint "last_four_digits_are_digits"',
	'Last four digits of a credit card number should be all digits'
);

select throws_ok (
	$$ update payment_account_card set last_four_digits = '1a34' where payment_account_id = 7 $$,
	'23514', 'new row for relation "payment_account_card" violates check constraint "last_four_digits_are_digits"',
	'Last four digits of a credit card number should be all digits'
);

select throws_ok (
	$$ update payment_account_card set last_four_digits = '12a4' where payment_account_id = 7 $$,
	'23514', 'new row for relation "payment_account_card" violates check constraint "last_four_digits_are_digits"',
	'Last four digits of a credit card number should be all digits'
);

select throws_ok (
	$$ update payment_account_card set last_four_digits = '123a' where payment_account_id = 7 $$,
	'23514', 'new row for relation "payment_account_card" violates check constraint "last_four_digits_are_digits"',
	'Last four digits of a credit card number should be all digits'
);

select throws_ok (
	$$ update payment_account_card set last_four_digits = '12345' where payment_account_id = 7 $$,
	'23514', 'new row for relation "payment_account_card" violates check constraint "last_four_digits_are_digits"',
	'Last four digits of a credit card number should be not have more than four digits'
);

select throws_ok (
	$$ update payment_account_card set last_four_digits = '123' where payment_account_id = 7 $$,
	'23514', 'new row for relation "payment_account_card" violates check constraint "last_four_digits_are_digits"',
	'Last four digits of a credit card number should be not have less than four digits'
);


select *
from finish();

rollback;

